from trac.core import *
from trac.web.chrome import add_stylesheet
from trac.web.href import Href
from trac.admin.api import IAdminPanelProvider
from model import LinkInfo
from api import GeneralLinkSyntaxProvider
import os

class LinkAdminPanel(Component):
    implements(IAdminPanelProvider)

    # IAdminPageProvider methods

    def get_admin_panels(self, req):
        if 'TRAC_ADMIN' in req.perm:
            yield 'wiki', 'Wiki System', 'link', 'Custom Links'

    def _set_message(self, req, cls, text):
        req.session['message_text'] = text
        req.session['message_class'] = cls

    def _get_message(self, req):
        return req.session.get('message_text', ''), \
               req.session.get('message_class', 'none')

    def _clear_message(self, req):
        dic = req.session
        if dic.has_key('message_text'):
            dic.pop('message_text')
        if dic.has_key('message_class'):
            dic.pop('message_class')

    def _notify(self, req, text):
        self._set_message(req, 'notify', text)

    def _error(self, req, text):
        self._set_message(req, 'error', 'ERROR: ' + text)

    def render_admin_panel(self, req, category, page, path_info):
    #def process_admin_request(self, req, category, page, name):
        req.perm.require('TRAC_ADMIN')
        self._notify(req, None)
        data = {
            'link':{'mode':None},
            }
        name = path_info
        # this page may called like
        # /proj/admin/wiki
        # /proj/admin/wiki/link
        # /proj/admin/wiki/link/name
        base_href = self.env.href.admin('wiki', 'link')

        # post back actions:
        #   add ... add and redirect to top
        #   modify ... modify and redirect self or renamed self
        #   remove ... redirect to top
        
        prov = GeneralLinkSyntaxProvider(self.env)
        prov.load();
        try:
            if name:
                link =  prov.get_link(name)
                if not link:
                    raise TracError('No such link name: ' + name)

                # prepare to modify
                data['link'] = link.get_hash()
                data['link']['mode'] = 'modify'
                if req.method == 'POST':
                    # change value
                    if req.args.get('save'):
                        # set values
                        prov.modify(name,
                                    req.args.get('expose') != None,
                                    req.args.get('disp'),
                                    req.args.get('url'))
                        self._notify(req, "link '%s' is modified'" % name)
                    # redirect to base
                    req.redirect(base_href)
            else:
                if req.method == 'POST':
                    if req.args.get('add'):
                        # add new entry
                        name = req.args.get('name')
                        expose = req.args.get('expose') != None
                        url = req.args.get('url')
                        disp = req.args.get('disp')
                        data['link'] = {'mode': 'add',
                                        'name': name,
                                        'expose': expose,
                                        'disp': disp,
                                        'url': url}
                        if expose:
                            data['link']['checked'] = "checked='checked'"
                        prov.add(name, expose, disp, url)
                        self._notify(req, "link '%s' is added'" % name)
                    elif req.args.get('remove') and req.args.get('sel'):
                        # Remove components
                        sel = req.args.get('sel')
                        sel = isinstance(sel, list) and sel or [sel]
                        if not sel:
                            raise TracError, 'No item selected'
                        for name in sel:
                            prov.delete(name)
                        self._notify(req, 'deleted %d links' % len(sel))
                    req.redirect(base_href)

        except Exception, e:
            self._error(req, str(e))
        
        # render current links
        links = []
        href = Href(base_href)
        for link in prov.get_links():
            h = link.get_hash()
            h['href'] = href(link.get_name())
            links.append(h)
        data['links'] = links
        # prepare notify message
        message_text, message_class = self._get_message(req)
        data['message'] = {
            'text': message_text,
            'class': message_class
        }
        self._clear_message(req)
        add_stylesheet(req, 'admin/css/admin_link.css')
        return 'admin_link.html', {'admin': data}
